# Author : Oceane DELANDRE
# Date : 07-2023
# Language : Bash
# Description : Semi-automated assembly-processing pipeline designed for Plasmodium falciparum. Input : Nanopore fastq. Output : assembly.
# Publication : Delandre et al. Long-read sequencing and de novo genome assembly pipeline of two Plasmodium falciparum clones (Pf3D7, PfW2) using only the PromethION sequencer from Oxford Nanopore Technologies without whole genome amplification
###
### Hardware description & software architecture
###

# Script was designed to run on a 48 CPU machine, OS Ubuntu 22.04
# Script can be run with lower CPU but script could become time-consuming.
# All software and tools were downloaded through conda (Anaconda3).
# Be sure to activate the right conda environment for each software you call.

###
### Assembly-processing pipeline
###

##
## Pre-processing
##

# Nanoplot - checking reads quality and estimating filtering thresholds
	NanoPlot -t 15 --fastq input.fastq -o output_nanoplot --title 'x' --plots hex dot
	(x = title to be specified)


# Minimap - extraction of reads of interest - decontaminating
	minimap2 -ax map-ont input1.fasta input2.fastq.gz | samtools fastq -n -f 4 - > clean.fastq
	# input1.fasta is a supposed contaminant reference genome (e.g. human)
	# input2.fastq is fastq reads from Nanopore sequencer
	# NB: this step can be repeated for as many contaminant as expected.

# Filtlong - filtering reads - retaining longest reads of highest quality 
	filtlong --min_length 2000 --keep_percent 90 --target_bases 20000000000 clean.fastq.gz | bgzip > output.filtlong.fastq.gz
	# Threshold to be adjusted with Nanoplot output: "--min_length" "--target_bases"

# Minimap & samtools - extracting reads of interest only
	minimap2 -ax map-ont --sam-hit-only genome_reference.fasta input.filtlong.fastq.gz > output.filtlong.sam
	samtools sort output.filtlong.sam > output.filtlong.bam
		# eliminate any contaminant by aligning reads to a very close reference genome
		# do not run these commands if your clone is too far from chosen reference.	
	
	# To remove reads that have not been mapped (lightens files)
	samtools view -F 4 -F 256 -F 512 -F 1024 -b -o output_samtoolsview.bam input.filtlong.bam
	
	# To visualise with IGV software which requires an index:
	samtools index output_samtoolsview.bam

##
## Raw assembly processing
##

# Converting fastq to fasta
	sed -n '1~4s/^@/>/p;2~4p' input.fastq.gz > output.fasta

# Flye - raw assembly
	flye --nano-raw output.filtlong.fasta --genome-size 23m --threads 20 --asm-coverage 35 --debug --out-dir output_flye 
	#Threshold to be adjusted according to sequencing results: "--asm-coverage" 

# Quast - assembly quality check
	quast input_contig.fasta -r input-reference.fasta --threads 10 --labels "title" -o output_quast --glimmer --nanopore input.filtlong.fastq.gz 

##
## Post-processing correction
##

# Bedtools - correcting misassembly events
	# Step 1 : call regions of interest in order from a bed formatted files (to build from Quast output)
	bedtools getfasta -s -fi assembly_flye.fasta -bed input_regioncalled.bed -fo output.fasta

	# Step 2 : delete the first line of the generated file
	sed "1d"  output_step1_in_input.fasta > output2.fasta

	# Step 3 : concatenate all files 
	cat output2.fasta output3.fasta > chromosome_1.fasta 

	# Step 4 : add the chromosome name to the first line of the file 
	sed -i 1i'>chromosome-1' chromosome_1.fasta

	# Step 5 : perform quality control with Quast 
	quast chromosome_1.fasta -r reference_sequence.fa --threads 45 --labels "Chromosome 1 assembly control " -o chromosome1_quast  --nanopore input.filtlong.fastq.gz 

	# Step 6 : concatenate all chromosome in a complete genome file
	cat chromosome1.fasta chromosome2.fasta > genome_complet.fasta 

# VCF - polishing the genome
	#Step 1 : index the fasta sequence used
	samtools faidx file.fasta 
	
	#Step 2 : call variants
	minimap2 -ax map-ont --sam-hit-only genome_complet.fasta input.filtlong.fastq.gz > genome_complet_alignmnent.sam
	samtools sort genome_complet_alignmnent.sam > genome_complet_alignmnent.bam
	bcftools mpileup --max-depth 800 --threads 35 -f genome_complet.fasta  genome_complet_alignmnent.bam | bcftools call -v -c  -o unfiltered_variants --ploidy 1
		#threshold to be adjusted: "--max-depth", "--threads"

	#Step 3 : filter true variations from false positive
	bcftools view unfiltered_variants -o filtered_variant.vcf -i '((DP4[1]+DP4[2])<(DP4[3]+DP4[4]) && (DP4[3]+DP4[4])>40)'
	#a quality filter can be added with command: "&& %QUAL>=50"
	
	#Step 4: generate consensus sequence (to immplement reported variations into the genome sequence)
	bgzip -c filtered_variant.vcf > filtered_variant.vcf.gz  
	bcftools index filtered_variant.vcf.gz
	cat genome_complet.fasta | bcftools consensus filtered_variant.vcf.gz > genome_complet_vcf1.fasta

# NB: run the VCF steps as long as the VCF content (i.e.: number of mutations) keep decreasing. Ideally, VCF should become void.

###
### Other processing steps that are not implemented in this script
###

# Assembly completion check and chromosome annotations with BUSCO (run with Galaxy = https://usegalaxy.org/ )
	# no command line

# Mitochondrial and apicoplastic annotations with Prokka (run with Galaxy = https://usegalaxy.org/ )
	# no command line

# Skeleton building - CLC Genomics Wb7 (online tool) 
	# no command line 

###
### End of script
###