#Author : Jean-Marie Loreau
#Date : 07-2023
#Language : R
#Description : BUSCO results analysis and Venn diagram creation
#Publication : Delandre et al. Long-read sequencing and de novo genome assembly pipeline of two Plasmodium falciparum clones (Pf3D7, PfW2) using only the PromethION sequencer from Oxford Nanopore Technologies without whole genome amplification

# library import
library(vcfR)
library(tidyr)
library(dplyr)
library(sqldf)
library(openxlsx)
library(ggvenn)
library(stringr)
library(ggplot2)
library(hrbrthemes)
library(cowplot)


# input BUSCO file for reference genome
busco_ref <- read.csv("busco_ref.csv", sep=";")
table(busco_ref$Status)

# input BUSCO file for the Pf3D7 and PfW2 clone
busco_3d7 <- read.csv("busco3d7.csv", sep=";")
busco_w2 <- read.csv("buscow2.csv", sep=";")
table(busco_3d7$Status)
table(busco_w2$Status)

# table of references
busco_ref_tab <- busco_ref %>% filter(Sequence!="") %>% 
  mutate(Chromosome = ifelse(Sequence=="Apicoplast","Ap",as.numeric(str_extract(Sequence,"(?<=\\_)\\d+")))) %>%
  select(-Status, -Strand, -Score, -Length, -Sequence, -Gene.Start, -Gene.End)
busco_3D7_tab <- busco_3d7 %>% filter(Sequence!="") %>%
  mutate(Chromosome = ifelse(Sequence=="Apicoplast","Ap",as.numeric(str_extract(Sequence,"(?<=\\_)\\d+")))) %>%
  select(-Status, -Strand, -Score, -Length, -Sequence, -Gene.Start, -Gene.End)
busco_W2_tab <- busco_w2 %>% filter(Sequence!="") %>%
  mutate(Chromosome = ifelse(Sequence=="Apicoplast","Ap",as.numeric(str_extract(Sequence,"(?<=\\_)\\d+")))) %>%
  select(-Status, -Strand, -Score, -Length, -Sequence, -Gene.Start, -Gene.End)
busco <- rbind(busco_ref_tab, busco_3D7_tab, busco_W2_tab)
busco <- unique(busco)

# joint Buscoref, Busco3D7 et BuscoW2 on busco id
busco_ref_statut <- busco_ref %>% rename(StatusRef = Status) %>% select(Busco.id, StatusRef)
busco_3d7_statut <- busco_3d7 %>% rename(Status3d7 = Status) %>% select(Busco.id, Status3d7)
busco_w2_statut <- busco_w2 %>% rename(StatusW2 = Status) %>% select(Busco.id, StatusW2)
buscoRef3d7W2 <- busco_ref_statut %>% left_join(busco_3d7_statut)
buscoRef3d7W2 <- buscoRef3d7W2  %>% left_join(busco_w2_statut)
busco3d7W2comp <- unite(buscoRef3d7W2, comp, StatusRef, Status3d7, StatusW2)
buscoRef3d7W2 <- buscoRef3d7W2 %>% left_join(busco3d7W2comp)
table(buscoRef3d7W2$comp, useNA = 'always')

# comparison tables
CCC <- busco3d7W2comp %>% filter(comp == "Complete_Complete_Complete") %>% left_join(busco)
CCF <- busco3d7W2comp %>% filter(comp == "Complete_Complete_Fragmented") %>% left_join(busco)    
CCM <- busco3d7W2comp %>% filter(comp == "Complete_Complete_Missing") %>% left_join(busco)
CFC <- busco3d7W2comp %>% filter(comp == "Complete_Fragmented_Complete") %>% left_join(busco)
CFF <- busco3d7W2comp %>% filter(comp == "Complete_Fragmented_Fragmented") %>% left_join(busco)
CFM <- busco3d7W2comp %>% filter(comp == "Complete_Fragmented_Missing") %>% left_join(busco)
CMC <- busco3d7W2comp %>% filter(comp == "Complete_Missing_Complete") %>% left_join(busco)
CMF <- busco3d7W2comp %>% filter(comp == "Complete_Missing_Fragmented") %>% left_join(busco)
CMM <- busco3d7W2comp %>% filter(comp == "Complete_Missing_Missing") %>% left_join(busco)
FFC <- busco3d7W2comp %>% filter(comp == "Fragmented_Fragmented_Complete") %>% left_join(busco)
FFF <- busco3d7W2comp %>% filter(comp == "Fragmented_Fragmented_Fragmented") %>% left_join(busco)
MCC <- busco3d7W2comp %>% filter(comp == "Missing_Complete_Complete") %>% left_join(busco)
MCM <- busco3d7W2comp %>% filter(comp == "Missing_Complete_Missing") %>% left_join(busco)
MMC <- busco3d7W2comp %>% filter(comp == "Missing_Missing_Complete") %>% left_join(busco)
MMM <- busco3d7W2comp %>% filter(comp == "Missing_Missing_Missing") %>% left_join(busco)

# creation of comparison excel file
tab <- list(CCC,CCF,CCM,CFC,CFF,CFM,CMC,CMF,CMM,FFC,FFF,MCC,MCM,MMC,MMM)
nom_tab <- c("CCC","CCF","CCM","CFC","CFF","CFM","CMC","CMF","CMM","FFC","FFF","MCC","MCM","MMC", "MMM")
write.xlsx(tab,"comp_busco.xlsx", sheetName = nom_tab)

# Venn diagram creation (need to requalify fragmented as Complete)
set1 <- buscoRef3d7W2$StatusRef
set2 <- buscoRef3d7W2$Status3d7
set3 <- buscoRef3d7W2$StatusW2
set1[set1=="Fragmented"] <- "Complete"
set2[set2=="Fragmented"] <- "Complete"
set3[set3=="Fragmented"] <- "Complete"
list_pres_gen <- seq(1, 3642)
list_pres_gen_set1 <- list_pres_gen[set1=="Complete"]
list_pres_gen_set2 <- list_pres_gen[set2=="Complete"]
list_pres_gen_set3 <- list_pres_gen[set3=="Complete"]
a <- list("3D7 réf" = list_pres_gen_set1,
          "3D7 assemblé" = list_pres_gen_set2,
          "W2" = list_pres_gen_set3)
ggvenn(a, fill_color = c("#0073C2FF", "#EFC000FF", "#CD534CFF"), stroke_size = 0.5, set_name_size = 4)
ggsave(filename = "busco_venn.png", device = "png")


