#Author : Jean-Marie Loreau
#Date : 07-2023
#Language : R
#Description : VCF results analysis
#Publication : Delandre et al. Long-read sequencing and de novo genome assembly pipeline of two Plasmodium falciparum clones (Pf3D7, PfW2) using only the PromethION sequencer from Oxford Nanopore Technologies without whole genome amplification 

# library import
library(vcfR)
library(tidyr)
library(dplyr)
library(sqldf)
library(openxlsx)
library(ggvenn)
library(stringr)
library(ggplot2)
library(hrbrthemes)
library(cowplot)


# reference input BUSCO file to identifie mutation localisation
busco_ref <- read.csv("busco_ref.csv", sep=";")

# VCF file input
vcf <- read.vcfR("filtered_variantsvcf3d7rename")

# VCF dataframe creation
vcf_tab <- as.data.frame(vcf@fix)

# VCF treatment
vcf_tab$Chromosome <- ifelse(vcf_tab$CHROM=="Apicoplast","Ap",as.numeric(str_extract(vcf_tab$CHROM,"(?<=\\_)\\d+")))
vcf_tab$Chromosome <- factor(vcf_tab$Chromosome, levels = c("1","2","3","4","5","6","7","8","9","10","11","12","13","14","Ap"))
vcf_tab$QUAL <- as.numeric(vcf_tab$QUAL)
vcf_tab$type <- ifelse(substr(vcf_tab$INFO,0,2) =="DP", "SNP", "INDEL")
vcf_tab$substitution <- unite(vcf_tab, substitution, REF, ALT, sep=" > ")$substitution
vcf_tab$DP <- as.numeric(str_extract(vcf_tab$INFO,"(?<=\\DP=)\\d+"))
vcf_tab <- vcf_tab %>% select(CHROM, Chromosome, POS, REF, ALT, QUAL, type, substitution, DP)

# graph mutation per chromosome
vcf_varbychr <- vcf_tab %>%
  group_by(Chromosome, type) %>%
  summarise(nb=n())

g1 <- ggplot(vcf_varbychr, aes(fill=type, y=nb, x=Chromosome)) + 
  geom_bar(position="stack", stat="identity") +
  theme_classic() +
  ylab("Variants") +
  theme(plot.title = element_text(hjust = 0.5)) +
  ggtitle("Number of variants")
ggsave(filename = "varbychr.png", device = "png")

# graph mutation per substitution
vcf_nbsubst <- vcf_tab %>% filter(type=="SNP") %>%
  mutate(top=str_length(ALT)) %>%
  filter(top==1) %>%
  group_by(substitution) %>%
  summarise(nb=n())

g2 <- ggplot(vcf_nbsubst, aes(y=nb, x=substitution)) + 
  geom_bar(position="stack", stat="identity", fill = "#FF6666") +
  theme_classic() +
  xlab("Substitution") +
  ylab("Frequence") +
  theme(plot.title = element_text(hjust = 0.5)) +
  ggtitle("Number of SNPs substitutions") + 
  theme(axis.text.x = element_text(angle = 45, hjust = 1))
ggsave(filename = "nbsubst.png", device = "png")

# depth vs. quality graph
vcf_meanDPQG <- vcf_tab %>%
  filter(type=="SNP") %>%
  group_by(Chromosome) %>%
  mutate(mean_DEPTHs = mean(DP, na.rm = TRUE)) %>%
  mutate(mean_QUALs = mean(QUAL, na.rm = TRUE)) %>%
  select(Chromosome, mean_DEPTHs, mean_QUALs) %>%
  distinct()

g3 <- ggplot(vcf_meanDPQG, aes(x=mean_DEPTHs, y=mean_QUALs, color=Chromosome)) + 
  geom_point(size=5) +
  theme_classic() +
  theme(plot.title = element_text(hjust = 0.5)) +
  ggtitle("QUAL vs DEPTH")
ggsave(filename = "qvsd.png", device = "png")

# multi-graph display
g4 <- plot_grid(g1, g2, g3, labels = c("A", "B", "C"), align = "hv")
save_plot("./multi.png", g4, base_height=4, ncol=2, nrow=2)

# Busco information added
vcf_tab_busco <- sqldf("select * from vcf_tab left join busco_ref 
                on (vcf_tab.CHROM = busco_ref.Sequence and 
                       vcf_tab.POS >= busco_ref.'Gene.Start' and vcf_tab.POS <= busco_ref.'Gene.End')")
vcf_tab_busco <- vcf_tab_busco %>% filter(!is.na(Busco.id)) %>%
  select(Chromosome, POS, REF, ALT, type, substitution, Busco.id, Status, Gene.Start, Gene.End, OrthoDB.url, Description)
descriptif_gene_mut <- vcf_tab_busco %>% 
  group_by(Description) %>%
  summarise(nb=n()) %>%
  arrange(desc(nb))

# output Excel file
tab <- list(vcf_varbychr,vcf_nbsubst,vcf_meanDPQG,vcf_tab_busco,descriptif_gene_mut)
nom_tab <- c("varbychr","nbsubst","meanDPQG","tab_busco","gene_mut")
write.xlsx(tab,"analyse_vcf.xlsx", sheetName = nom_tab)

